type ligne = { num   : int;    (* numero de la ligne *)
              y     : float;}  (* ordonnee de la ligne *)

(* dimension du plan *)
let x_min = (0.280500)
let x_max = (0.282500)
let y_min = (-0.011500)
let y_max = (-0.009500)
(* resolution  *)
let step  = (0.000002)

(* nombre de colonnes *)
let n_x   = int_of_float ((x_max -. x_min) /. step)
(* nombre de lignes *)
let nb_lignes = int_of_float ((y_max -. y_min) /. step)
(* nombre de threads *)
let nb_worker = 8

(* conditions de convergence/divergence *)
let break_value_d  = 900000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000.
let max_iterations = 4000

(* suite complexe pour le calcul de l'espace de mandelbrot *)
let rec suite base var i =
 if i > max_iterations then
   max_iterations
 else
   let res = Complex.add (Complex.mul var var) base in
   if Complex.norm2 res >= break_value_d then
     i
   else
     suite base res (i+1)

(* teste la convergence de la suite pour le point (x,y) *)
let process_point x y =
 suite
   { Complex.re = x;
     Complex.im = y;}
   Complex.zero 0

(* teste la convergence pour une ligne *)
let process_line line =
  let rec fill_list x l =
   if x >= n_x then l
   else
     fill_list
       (x+1)
       ((process_point (x_min +. (float_of_int x) *. step) line.y) :: l)
  in
 List.rev (fill_list 0 [])

(* genere structure ligne associee a l'ordonne n *)
let gen_ligne n =
 {num = n; y = y_min +. (float_of_int n) *. step}

(* compteur de taches *)
let get_n =
  let m = Mutex.create () in
  let c = ref 0 in
   fun () ->
     begin
    Mutex.lock m;
       let ret = !c in
      c := ret + 1;
      Mutex.unlock m;
      ret
     end

(* stockage des resultats *)
let resultats = Array.make nb_lignes []

let xxx = Mutex.create ()

(* corps des threads *)
let rec worker () =
  let n = get_n () in
   if n >= nb_lignes then
     ()
   else
     begin
       Mutex.lock xxx;
       Array.set resultats n (process_line (gen_ligne n));
       Mutex.unlock xxx;
       worker ()
     end

let start _ =
 Thread.create worker ()

(* affichage d'une ligne *)
let draw_line n l =
  let rec draw_list l i =
   match l with
     | [] -> ()
     | hd :: tl ->
	 let c =
        if hd = max_iterations then
          (Graphics.rgb 0 0 0)
        else
          let r = ((hd * 200) / max_iterations) + 55 in
          let g = ((hd * 255) / max_iterations) in
          let b = ((hd * 255) / max_iterations) in
        (Graphics.rgb r g b)
	 in
        Graphics.set_color c;
        Graphics.plot i n;
        draw_list tl (i + 1)
  in
   draw_list l 0;
   Graphics.synchronize()

let draw_all () =
 Array.iteri draw_line resultats

let _ =
 Graphics.open_graph " 1000x1000+50-50";
 Graphics.auto_synchronize false;
 let threads =
   Array.init
     nb_worker
     (fun _ -> Thread.create worker ())
 in
   Array.iter Thread.join threads;
   draw_all ();
   print_endline "affichage reussit";
   ignore (input_char stdin)
